import { LitElement, html, css } from 'lit';
import { customElement } from 'lit/decorators.js';

type RSSItem = {
  title: string;
  link: string | null;
}

@customElement('mosaic-rss')
export class RSS extends LitElement {
  rssItems: RSSItem[] = [];
  rssTitle = '';
  rssLink: string | null = null;

  constructor() {
    super();
    this.pullValueFromBackend();
  }

  async pullValueFromBackend() {
    const urlParams = new URLSearchParams(window.location.search);

    const url = 'https://RSS_API_HOST/';
    const options = {
      method: 'POST',
      headers: {
        'content-type': 'application/json',
        Origin: 'RSS_ORIGIN',
        'X-Requested-With': 'RSS_ORIGIN',
        'X-RapidAPI-Key': 'RSS_API_KEY',
        'X-RapidAPI-Host': 'RSS_API_HOST'
      },
      body: JSON.stringify({
        url: urlParams.get('rss'),
      }),
    };

    try {
      const allItems: RSSItem[] = [];
      const response = await fetch(url, options);
      const result = await response.text();
      const rss = (new DOMParser()).parseFromString(result.trim(), 'text/xml');
      const channel = rss.getElementsByTagName('channel')[0] ?? null;

      for (let channelI = 0; channelI < channel?.children?.length; channelI++) {
        const channelChild = channel?.children?.item(channelI);

        if (channelChild?.tagName === 'title') {
          this.rssTitle = channelChild?.textContent ?? 'Undefined';
        } else if (channelChild?.tagName === 'link') {
          this.rssLink = channelChild?.textContent ?? null;
        } else if (channelChild?.tagName === 'item') {
          const items = channelChild?.children;
          let itemTitle = '';
          let itemLink = null;

          for (let itemsI = 0; itemsI < items?.length; itemsI++) {
            const item = items.item(itemsI);
            if (item?.tagName === 'title') {
              itemTitle = item?.textContent ?? 'Undefined';
            } else if (item?.tagName === 'link') {
              itemLink = item?.textContent ?? null;
            }
          }

          allItems.push({
            title: itemTitle,
            link: itemLink,
          });
        }
      }

      this.rssItems = allItems;
      this.requestUpdate();

    } catch (error) {
      console.error(error);
    }

  }

  static override styles = [
    css`
      .gauge-body {
        display: flex;
        flex-direction: column;
        justify-content: space-between;
        align-content: space-between;
        text-align: center;
      }

      .gauge-label {
        font-weight: bold;
      }
    `
  ]
  override render() {
    return html`
      <div class="gauge-body">
        ${this.rssItems?.length > 0
        ? html`
          <div class="gauge-label">RSS Feed for ${this.rssTitle}${this.rssLink ? html` <a href="${this.rssLink}" target="_blank" style="text-decoration: none">&#x1F517;</a>` : ''}</div>
                    ${this.rssItems.map(rssItem => html`
                      <div>
                        <div>${rssItem.link ? html`<a href="${rssItem.link}" target="_blank">${rssItem.title}</a>` : rssItem.title}</div>
                      </div>
                    `)}
          `
        : html`<div class="gauge-label">There was no valid RSS feed found.</div>`
      }
        
      </div>
`;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'mosaic-rss': RSS;
  }
}